@@ -6,23 +6,34 @@ sentry_webhooks.plugin
 :license: BSD, see LICENSE for more details.
 """
 
+import logging
+import sentry_webhooks
+
+from django.conf import settings
 from django import forms
 from django.utils import simplejson
-from sentry.utils.safe import safe_execute
 from django.utils.translation import ugettext_lazy as _
-from sentry.plugins import Plugin
 
-import sentry_webhooks
-import urllib2
+from sentry.plugins.bases.notify import NotificationPlugin
+from sentry.http import safe_urlopen, is_valid_url
+from sentry.utils.safe import safe_execute
 
 
 class WebHooksOptionsForm(forms.Form):
-    urls = forms.CharField(label=_('Repository Name'),
-        widget=forms.Textarea(attrs={'class': 'span6', 'placeholder': 'https://getsentry.com/callback/url'}),
+    urls = forms.CharField(
+        label=_('Callback URLs'),
+        widget=forms.Textarea(attrs={
+            'class': 'span6', 'placeholder': 'https://getsentry.com/callback/url'}),
         help_text=_('Enter callback URLs to POST new events to (one per line).'))
 
+    def clean_url(self):
+        value = self.cleaned_data.get('url')
+        if not is_valid_url(value):
+            raise forms.ValidationError('Invalid hostname')
+        return value
 
-class WebHooksPlugin(Plugin):
+
+class WebHooksPlugin(NotificationPlugin):
     author = 'Sentry Team'
     author_url = 'https://github.com/getsentry/sentry'
     version = sentry_webhooks.VERSION
@@ -37,6 +48,9 @@ class WebHooksPlugin(Plugin):
     conf_title = title
     conf_key = 'webhooks'
     project_conf_form = WebHooksOptionsForm
+    timeout = getattr(settings, 'SENTRY_WEBHOOK_TIMEOUT', 3)
+    logger = logging.getLogger('sentry.plugins.webhooks')
+    user_agent = 'sentry-webhooks/%s' % version
 
     def is_configured(self, project, **kwargs):
         return bool(self.get_option('urls', project))
@@ -51,27 +65,27 @@ class WebHooksPlugin(Plugin):
             'level': group.get_level_display(),
             'culprit': group.culprit,
             'message': event.message,
+            'url': group.get_absolute_url(),
         }
-        data['event'] = event.data or {}
+        data['event'] = dict(event.data or {})
         return data
 
     def get_webhook_urls(self, project):
-        return filter(bool, self.get_option('urls', project).strip().splitlines())
+        urls = self.get_option('urls', project)
+        if not urls:
+            return ()
+        return filter(bool, urls.strip().splitlines())
 
     def send_webhook(self, url, data):
-        req = urllib2.Request(url, data)
-        req.add_header('User-Agent', 'sentry-webhooks/%s' % self.version)
-        req.add_header('Content-Type', 'application/json')
-        resp = urllib2.urlopen(req)
-        return resp
-
-    def post_process(self, group, event, is_new, is_sample, **kwargs):
-        if not is_new:
-            return
-
-        if not self.is_configured(group.project):
-            return
+        return safe_urlopen(
+            url=url,
+            data=data,
+            timeout=self.timeout,
+            user_agent=self.user_agent,
+            headers=(('Accept-Encoding', 'gzip'), ('Content-type', 'application/json')),
+        )
 
+    def notify_users(self, group, event, fail_silently=False):
         data = simplejson.dumps(self.get_group_data(group, event))
         for url in self.get_webhook_urls(group.project):
             safe_execute(self.send_webhook, url, data)
